==> imwiki-0.6.0.tar.gz: imwiki-0.6.0/imLib/imParser/WikiParser.py <==
# -*- coding: utf8 -*-
import re
import imLib.Category as Category

class TableParser:
	def __init__(self, sep='||'):
		self.sep = sep
	def feed(self, rawText):
		self.rawText = rawText
	def parse(self):
		self.rowList = []
		for line in self.rawText.strip().splitlines():
			colList = line.strip().split(self.sep)[1:-1]
			L = []
			for col in colList:
				align, col = self.getAlign(col)
				if align:
					L.append((align, col))
				else:
					L.append(col)
			self.rowList.append(L)
		return self.rowList
	def getAlign(self, col):			
		if len(col) < 2:
			return '', col
		if col[0:2] in ['><', '<>']:
			align = 'center'
			col = col[2:]
		elif col[0] == '<':
			align = 'left'
			col = col[1:]
		elif col[0] == '>':
			align = 'right'
			col = col[1:]
		else:
			align = ''
		return align, col

class IndentParser:
	def __init__(self):
		pass
	def feed(self, rawText):
		self.rawText = rawText
	def parse(self):
		lines = self.rawText.splitlines() # FIXME lost last \n
		lastIndent = 0
		lastIndentList = []
		self.result = []
		for i in range(len(lines)):
			line = lines[i]
			curIndent , contents = self.split(line)
			indentCount = 0
			if curIndent > lastIndent:
				indentCount = indentCount + 1
			while curIndent < lastIndent:
				indentCount = indentCount - 1
				lastIndent = lastIndentList.pop()
			if curIndent <> lastIndent:
				lastIndentList.append(lastIndent)
			lastIndent = curIndent
			self.result.append((indentCount, contents+'\n'))
		self.result.append((len(lastIndentList) * -1 ,''))
		return self.result

	def split(self, line):
		indentLevel = 0
		for c in line:
			if c == ' ': indentLevel = indentLevel + 1
			else: break
		return indentLevel, line[indentLevel:]

validMarkUpList = {
	'heading':('strike', 'underline', 'internalLink', 'camelCase'),
	'pre':(),
	'tt':(),
	'strong':('strike', 'underline', 'internalLink', 'camelCase', 'externalLink'), 
	'strongEm':('strike', 'underline', 'internalLink', 'camelCase', 'externalLink'), 
	'em':('strike', 'underline', 'internalLink', 'camelCase', 'externalLink'), 
	'strike':('strongEm', 'strong', 'em', 'underline', 'internalLink', 'camelCase', 'externalLink'),
	'underline':('strongEm', 'strong', 'em', 'strike', 'internalLink', 'camelCase', 'externalLink'),
	'horizon':(),
	'table':('strong', 'strongEm', 'em', 'strike', 'underline', 'internalLink', 'camelCase', 'externalLink', 'macro'),
	'list':('listitem'),
	'indent':('strong', 'strongEm', 'em', 'strike', 'underine', 'listitem', 'internalLink', 'camelCase', 'externalLink', 'macro', 'br'),
	'listitem':('strong', 'strongEm', 'em', 'strike', 'underline', 'internalLink', 'camelCase', 'externalLink', 'macro'),
	'macro':(),
	'internalLink':(),
	'camelCase':(),
	'externalLink':(),
	'br':(),
}
class Parser:
	def __init__(self, formatter):
		self.formatter = formatter

		patterns = r"""
(^(?P<heading_marker>={1,6})( +)(?P<heading>[^\r\n]+?)( +)(?P=heading_marker)$[\r]?[\n]?)
(^\{{3}$(?P<pre>.+?)^\}{3} *?$)
(\{{3}(?P<tt>.+)\}{3})
('{5}(?P<strongEm>[^'].+?)'{5})
('{3}(?P<strong>[^'].+?)'{3})
('{2}(?P<em>[^'].+?)'{2})
(~{2}(?P<strike>[^~].+?)~{2})
(_{2}(?P<underline>[^_].+?)_{2})
(\^{2}(?P<superscript>[^_].+?)\^{2})
(`{2}(?P<subscript>[^_].+?)`{2})
(?P<horizon>^-{4,}[\r\n]{0,1}$)
(?P<table>(^(\|\|.*?)+\|\|$[\r]?[\n]?)+)
(?P<list>(^ +(\*|(\d+\.)|([a-zA-Z]\.)) +[^\r\n]+$[\r]?[\n]?)+)
(^ *(\*|(\d+\.)|([a-zA-Z]\.)) (?P<listitem>[^\r\n]+))
(?P<indent>(^ +[^\r\n]+$[\r\n]*)+)
(\[\[(?P<macro>([^\[\r\n]+?))\]\])
(\[(?P<internalLink>([^\[]+?))\])
(?P<camelCase>([A-Z][a-z]+){2,})
(?P<externalLink>http://[^\s]+)
(?P<br>[\n])
"""

		patterns = patterns.strip().replace('\n', '|')
		self.rules = re.compile(patterns, re.DOTALL|re.MULTILINE)


	def feed(self, rawText):
		self.rawText = rawText.replace('\r\n', '\n')

	def parse(self):
		self.stackOfopenedMarkUp = []
		self.parsedText = self.rawText
		self.parsedText = self.rules.sub(self.replaceAgent, self.parsedText)
		return self.parsedText

	# replace methods
	def replaceAgent(self, match):
		D = match.groupdict()
		del D['heading_marker']
		type, matchedStr = filter(lambda group: group[1], D.items())[0]
		replFunc = getattr(self, '_repl%s' % type.capitalize())
		if self.stackOfopenedMarkUp and type not in validMarkUpList[self.stackOfopenedMarkUp[-1]]:
			return match.group()
		insertTheMatchObjList = ['heading', 'listitem'] #FIXME my name is ...-_-
		self.stackOfopenedMarkUp.append(type)
		if type in insertTheMatchObjList:
			result = replFunc(match)
		else:
			result = replFunc(matchedStr)
		self.stackOfopenedMarkUp.pop()
		return result

	def _replStrong(self, matchedStr):
		return self.formatter.strong(self.rules.sub(self.replaceAgent, matchedStr))
	def _replEm(self, matchedStr):
		return self.formatter.em(self.rules.sub(self.replaceAgent, matchedStr))
	def _replStrongem(self, matchedStr):
		return self.formatter.strongEm(self.rules.sub(self.replaceAgent, matchedStr))
	def _replStrike(self, matchedStr):
		return self.formatter.strike(self.rules.sub(self.replaceAgent, matchedStr))
	def _replUnderline(self, matchedStr):
		return self.formatter.underline(self.rules.sub(self.replaceAgent, matchedStr))
	def _replSuperscript(self, matchedStr): # FIXME testcode
		return self.formatter.superscript(self.rules.sub(self.replaceAgent, matchedStr))
	def _replSubscript(self, matchedStr): # FIXME testcode..
		return self.formatter.subscript(self.rules.sub(self.replaceAgent, matchedStr))
	def _replHorizon(self, matchedStr): # FIXME
		return self.formatter.horizon(matchedStr)
	def _replPre(self, matchedStr):
		return self.formatter.pre(matchedStr)
	def _replHeading(self, match):
		title = match.groupdict()['heading']
		level = len(match.groupdict()['heading_marker'])
		return self.formatter.heading(title, level)

	def _replTable(self, matchedStr):
		tableParser = TableParser()
		tableParser.feed(matchedStr)
		rowList = tableParser.parse()
		return self.formatter.table(rowList)

	def _replList(self, matchedStr):
		indentParser = IndentParser()
		indentParser.feed(matchedStr)
		L = []
		parsedList = indentParser.parse()

		newParsedList = []
		for indentLevel, parsedText in parsedList[:-1]:
			parsedText = self.rules.sub(self.replaceAgent, parsedText)
			newParsedList.append((indentLevel, parsedText))
		newParsedList.append(parsedList[-1])
		return self.formatter.List(newParsedList)

	def _replIndent(self, matchedStr): # FIXME
		indentParser = IndentParser()
		indentParser.feed(matchedStr)
		L = []
		parsedList = indentParser.parse()

		newParsedList = []
		for indentLevel, parsedText in parsedList[:-1]:
			parsedText = self.rules.sub(self.replaceAgent, parsedText)
			newParsedList.append((indentLevel, parsedText))
		newParsedList.append(parsedList[-1])
		return self.formatter.indent(newParsedList)

	def _replListitem(self, match):
		type, matchedStr = filter(lambda group: group[1], match.groupdict().items())[0]
		stackLength = len(self.stackOfopenedMarkUp)
		if stackLength < 2 or (stackLength >= 2 and self.stackOfopenedMarkUp[-2] not in  ['indent', 'list']):
			return self.rules.sub(self.replaceAgent, match.group())
		return self.formatter.listitem(self.rules.sub(self.replaceAgent, matchedStr))

	def _replTt(self, matchedStr):
		return self.formatter.tt(matchedStr)

	def _replInternallink(self, matchedStr):
		if matchedStr.startswith('http://'):  #FIXME 지져분하다. 정리해라. 
			firstSpace = matchedStr.find(' ')
			if firstSpace > 0:
				url = matchedStr[:firstSpace]
				linkName = matchedStr[firstSpace+1:]
				return self.formatter.externalLink(url, linkName)
			return self.formatter.externalLink(matchedStr)

		L = matchedStr.split(Category.sep)
		pageName = L[-1]
		categoryPath = Category.sep.join(L[:-1])
		isAbsolutePath = matchedStr and matchedStr[0] == '/'
		return self.formatter.internalLink(categoryPath, pageName, isAbsolutePath)
	def _replCamelcase(self, matchedStr):
		return self._replInternallink(matchedStr)
	def _replExternallink(self, matchedStr):
		return self.formatter.externalLink(matchedStr)

	def _replMacro(self, matchedStr):
		return self.formatter.macro(matchedStr)
		return matchedStr
		L = matchedStr.split('(')
		if len(L) > 1:
			macroName = L[0]
			macroArgs = L[-1]
		return matchedStr
	def _replBr(self, matchedStr):
		if 'pre' not in self.stackOfopenedMarkUp:
			return '<br />'
		return '\n'

class EchoFormatter: # FOR UNITTEST
	def __init__(self):
		self._stack = []
	def echo(self, *args):
		s = ' : '.join([str(arg) for arg in args])
		key = self._stack.pop()
		return '<%s>%s</%s>' % (key, s, key)
	def __getattr__(self, key):
		self._stack.append(key)
		return self.echo

==> imwiki-0.6.1.tar.gz: imwiki-0.6.1/imLib/imParser/WikiParser.py <==
# -*- coding: utf8 -*-
import re
import string
import imLib.Category as Category

class TableParser:
	def __init__(self, sep='||'):
		self.sep = sep
	def feed(self, rawText):
		self.rawText = rawText
	def parse(self):
		self.rowList = []
		for line in self.rawText.strip().splitlines():
			colList = line.strip().split(self.sep)[1:-1]
			L = []
			for col in colList:
				align, col = self.getAlign(col)
				if align:
					L.append((align, col))
				else:
					L.append(col)
			self.rowList.append(L)
		return self.rowList
	def getAlign(self, col):			
		if len(col) < 2:
			return '', col
		if col[0:2] in ['><', '<>']:
			align = 'center'
			col = col[2:]
		elif col[0] == '<':
			align = 'left'
			col = col[1:]
		elif col[0] == '>':
			align = 'right'
			col = col[1:]
		else:
			align = ''
		return align, col

class IndentParser:
	def __init__(self):
		pass
	def feed(self, rawText):
		self.rawText = rawText
	def parse(self):
		lines = self.rawText.splitlines() 
		lastIndent = 0
		lastIndentList = []
		self.result = []
		for i in range(len(lines)):
			line = lines[i]
			curIndent , contents = self.split(line)
			if curIndent > lastIndent:
				indentCount = 1
			elif curIndent < lastIndent:	
				indentCount, lastIndent = self.dedentCounter(curIndent, lastIndent, lastIndentList)
			else:
				self.insertToPrevContents(contents)
				continue
			if curIndent != lastIndent:
				lastIndentList.append(lastIndent)
			lastIndent = curIndent
			self.result.append((indentCount, contents+'\n'))
		return self.result
	def insertToPrevContents(self, contents):
		indentCount, prevContents = self.result[-1]
		self.result[-1] = (indentCount, prevContents + contents+'\n')

	def dedentCounter(self, curIndent, lastIndent, lastIndentList):
		indentCount = 0
		while curIndent < lastIndent:
			indentCount = indentCount - 1
			lastIndent = lastIndentList.pop()
		return indentCount, lastIndent

	def split(self, line):
		indentLevel = 0
		for c in line:
			if c == ' ': indentLevel = indentLevel + 1
			else: break
		return indentLevel, line[indentLevel:]

validMarkUpList = {
	'heading':('strike', 'underline', 'internalLink', 'camelCase'),
	'pre':(),
	'tt':(),
	'strong':('strike', 'underline', 'internalLink', 'camelCase', 'externalLink'), 
	'strongEm':('strike', 'underline', 'internalLink', 'camelCase', 'externalLink'), 
	'em':('strike', 'underline', 'internalLink', 'camelCase', 'externalLink'), 
	'strike':('strongEm', 'strong', 'em', 'underline', 'internalLink', 'camelCase', 'externalLink'),
	'underline':('strongEm', 'strong', 'em', 'strike', 'internalLink', 'camelCase', 'externalLink'),
	'horizon':(),
	'table':('strong', 'strongEm', 'em', 'strike', 'underline', 'internalLink', 'camelCase', 'externalLink', 'macro'),
	'list':('listitem'),
	'indent':('strong', 'strongEm', 'em', 'strike', 'underine', 'listitem', 'internalLink', 'camelCase', 'externalLink', 'macro', 'br'),
	'listitem':('strong', 'strongEm', 'em', 'strike', 'underline', 'internalLink', 'camelCase', 'externalLink', 'macro'),
	'macro':(),
	'internalLink':(),
	'camelCase':(),
	'externalLink':(),
	'br':(),
}
class Parser:
	def __init__(self, formatter):
		self.formatter = formatter

		patterns = r"""
(^(?P<heading_marker>={1,6})( +)(?P<heading>[^\r\n]+?)( +)(?P=heading_marker)$[\r]?[\n]?)
(^\{{3}$(?P<pre>.+?)^\}{3} *?$)
(\{{3}(?P<tt>.+)\}{3})
('{5}(?P<strongEm>[^'].+?)'{5})
('{3}(?P<strong>[^'].+?)'{3})
('{2}(?P<em>[^'].+?)'{2})
(~{2}(?P<strike>[^~].+?)~{2})
(_{2}(?P<underline>[^_].+?)_{2})
(\^{2}(?P<superscript>[^_].+?)\^{2})
(`{2}(?P<subscript>[^_].+?)`{2})
(?P<horizon>^-{4,}[\r\n]{0,1}$)
(?P<table>(^(\|\|.*?)+\|\|$[\r]?[\n]?)+)
(?P<list>(^ +(\*|(\d+\.)|([a-zA-Z]\.)) +[^\r\n]+$[\r]?[\n]?)+)
(^ *(\*|(\d+\.)|([a-zA-Z]\.)) (?P<listitem>[^\r\n]+))
(?P<indent>(^ +[^\r\n]+$[\r\n]*)+)
(\[\[(?P<macro>([^\[\r\n]+?))\]\])
(\[(?P<internalLink>([^\[]+?))\])
(?P<camelCase>/?(([A-Z][a-z]+){2,}/?)+)
(?P<externalLink>http://[^\s]+)
(?P<br>[\n])
"""

		patterns = patterns.strip().replace('\n', '|')
		self.rules = re.compile(patterns, re.DOTALL|re.MULTILINE)


	def feed(self, rawText):
		self.rawText = rawText.replace('\r\n', '\n')

	def parse(self):
		self.stackOfopenedMarkUp = []
		self.parsedText = self.rawText
		self.parsedText = self.rules.sub(self.replaceAgent, self.parsedText)
		return self.parsedText

	# replace methods
	def replaceAgent(self, match):
		D = match.groupdict()
		del D['heading_marker']
		type, matchedStr = filter(lambda group: group[1], D.items())[0]
		replFunc = getattr(self, '_repl%s' % type.capitalize())
		if self.stackOfopenedMarkUp and type not in validMarkUpList[self.stackOfopenedMarkUp[-1]]:
			return match.group()
		insertTheMatchObjList = ['heading', 'listitem'] #FIXME my name is ...-_-
		self.stackOfopenedMarkUp.append(type)
		if type in insertTheMatchObjList:
			result = replFunc(match)
		else:
			result = replFunc(matchedStr)
		self.stackOfopenedMarkUp.pop()
		return result

	def _replStrong(self, matchedStr):
		return self.formatter.strong(self.rules.sub(self.replaceAgent, matchedStr))
	def _replEm(self, matchedStr):
		return self.formatter.em(self.rules.sub(self.replaceAgent, matchedStr))
	def _replStrongem(self, matchedStr):
		return self.formatter.strongEm(self.rules.sub(self.replaceAgent, matchedStr))
	def _replStrike(self, matchedStr):
		return self.formatter.strike(self.rules.sub(self.replaceAgent, matchedStr))
	def _replUnderline(self, matchedStr):
		return self.formatter.underline(self.rules.sub(self.replaceAgent, matchedStr))
	def _replSuperscript(self, matchedStr): # FIXME testcode
		return self.formatter.superscript(self.rules.sub(self.replaceAgent, matchedStr))
	def _replSubscript(self, matchedStr): # FIXME testcode..
		return self.formatter.subscript(self.rules.sub(self.replaceAgent, matchedStr))
	def _replHorizon(self, matchedStr): # FIXME
		return self.formatter.horizon(matchedStr)
	def _replPre(self, matchedStr):
		return self.formatter.pre(matchedStr)
	def _replHeading(self, match):
		title = match.groupdict()['heading']
		level = len(match.groupdict()['heading_marker'])
		return self.formatter.heading(title, level)

	def _replTable(self, matchedStr):
		tableParser = TableParser()
		tableParser.feed(matchedStr)
		rowList = tableParser.parse()
		return self.formatter.table(rowList)

	def _replList(self, matchedStr): # FIXME replList, replIndent 너무 비슷함
		def getListType(c):
			if c == 'i':
				return 'i'
				return 'lower-roman'
			elif c == 'I':
				return 'I'
				return 'upper-roman'
			elif c in string.lowercase:
				return 'a'
				return 'lower-alpha'
			elif c in string.uppercase:
				return 'A'
				return 'upper-alpha'
			elif c in string.digits:
				return '1'
				return 'decimal'
			return None # * => None
		indentParser = IndentParser()
		indentParser.feed(matchedStr)
		parsedList = indentParser.parse()
		L = []
		newParsedList = []
		for indentLevel, parsedText in parsedList:
			listType = getListType(parsedText[0])
			parsedText = self.rules.sub(self.replaceAgent, parsedText)
			newParsedList.append((indentLevel, listType, parsedText))
		return self.formatter.List(newParsedList)

	def _replIndent(self, matchedStr): # FIXME
		indentParser = IndentParser()
		indentParser.feed(matchedStr)
		L = []
		parsedList = indentParser.parse()
		newParsedList = []
		for indentLevel, parsedText in parsedList:
			parsedText = self.rules.sub(self.replaceAgent, parsedText)
			newParsedList.append((indentLevel, parsedText))
		return self.formatter.indent(newParsedList)

	def _replListitem(self, match):
		type, matchedStr = filter(lambda group: group[1], match.groupdict().items())[0]
		stackLength = len(self.stackOfopenedMarkUp)
		if stackLength < 2 or (stackLength >= 2 and self.stackOfopenedMarkUp[-2] not in  ['indent', 'list']): #  FIXME 왜 이런 구조지? 기억이..
			return self.rules.sub(self.replaceAgent, match.group())
		return self.formatter.listitem(self.rules.sub(self.replaceAgent, matchedStr))

	def _replTt(self, matchedStr):
		return self.formatter.tt(matchedStr)

	def _replInternallink(self, matchedStr):
		if matchedStr.startswith('http://'):  #FIXME 지져분하다. 정리해라. 
			firstSpace = matchedStr.find(' ')
			if firstSpace > 0:
				url = matchedStr[:firstSpace]
				linkName = matchedStr[firstSpace+1:]
				return self.formatter.externalLink(url, linkName)
			return self.formatter.externalLink(matchedStr)

		L = matchedStr.split(Category.sep)
		pageName = L[-1]
		categoryPath = Category.sep.join(L[:-1])
		isAbsolutePath = matchedStr and matchedStr[0] == '/'
		return self.formatter.internalLink(categoryPath, pageName, isAbsolutePath)
	def _replCamelcase(self, matchedStr):
		return self._replInternallink(matchedStr)
	def _replExternallink(self, matchedStr):
		return self.formatter.externalLink(matchedStr)

	def _replMacro(self, matchedStr):
		return self.formatter.macro(matchedStr)
		return matchedStr
		L = matchedStr.split('(')
		if len(L) > 1:
			macroName = L[0]
			macroArgs = L[-1]
		return matchedStr
	def _replBr(self, matchedStr):
		if 'pre' not in self.stackOfopenedMarkUp:
			return '<br />'
		return '\n'

class EchoFormatter: # FOR UNITTEST
	def __init__(self):
		self._stack = []
	def echo(self, *args):
		s = ' : '.join([str(arg) for arg in args])
		key = self._stack.pop()
		return '<%s>%s</%s>' % (key, s, key)
	def __getattr__(self, key):
		self._stack.append(key)
		return self.echo
